//===================================================================
// File Name : 2442lib.c
// Function  : S3C2410 PLL,Uart, LED, Port Init
// Date      : March 20, 2002
// Version   : 0.0
// History
//   0.0 : Programming start (February 20,2002) -> SOP
//===================================================================

#include "def.h"
#include "option.h"
#include "244X_addr.h"
#include "244X_lib.h"

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

//#define PreScaler  (PCLK/1000000-1)
#ifdef NO_USE_STANDARD_C_LIB
void DIV(U32 p1, U32 p2, DIV_T* result)
{
	U32 quot=0;
	if (p2==0) {
		result->quot = 0;
		result->rem = 0;
		return;
	}
	else if (p2==1) {
		result->quot = p1;
		result->rem = p1;
		return;
	}
	
	while (p1 > p2) {
		p1 -= p2;
		quot++;
	}

	result->quot = quot;
	result->rem = p1;
	return ;
}

void Memset(char *dst,  char src, unsigned int len)
{
	if (len &3 == 0 && ((unsigned)dst) & 3 == 0) {
		register U32 *d = (U32 *)dst;
		register U32 v = (unsigned)src | ((unsigned)src << 8) | ((unsigned)src << 16) | ((unsigned)src << 24);
		register U32 l = len / 4; 
		while (l--) {
			*d++ = v;
		}
	} else {
		while(len--) 
			*dst++ = src;
	}
}

void Memcpy(char *dst,  char *src, unsigned int len)
{
	if (len &3 == 0 && ((unsigned)dst) & 3 == 0 && ((unsigned)dst) & 3 == 0) {
		U32 *d = (U32 *)dst;
		U32 *s = (U32 *)src;
		U32 l = len / 4; 
		while (l--) {
			*d++ = *s++;
		}
	} else {
		while(len--) 
			*dst++ = *src++;
	}
}

#endif
//***************************[ SYSTEM ]***************************************************
//static int delayLoopCount = 400;

void Delay(int time)
{
      // time=0: adjust the Delay function by WatchDog timer.
      // time>0: the number of loop time
      // resolution of time is 100us.
    int i; //,adjust=0;
     for(i=0; i<1000; i++);
/*
    if(time==0)
    {
        time   = 200;
        adjust = 1;
        delayLoopCount = 400;
            //PCLK/1M,Watch-dog disable,1/64,interrupt disable,reset disable
        rWTCON = ((PCLK/1000000-1)<<8)|(2<<3); 
        rWTDAT = 0xffff;                              //for first update
        rWTCNT = 0xffff;                              //resolution=64us @any PCLK 
        rWTCON = ((PCLK/1000000-1)<<8)|(2<<3)|(1<<5); //Watch-dog timer start
    }
    for(;time>0;time--)
        for(i=0;i<delayLoopCount;i++);
    if(adjust==1)
    {
        rWTCON = ((PCLK/1000000-1)<<8)|(2<<3); //Watch-dog timer stop
        i = 0xffff - rWTCNT;                     //1count->64us, 200*400 cycle runtime = 64*i us
        delayLoopCount = 8000000/(i*64);         //200*400:64*i=1*x:100 -> x=80000*100/(64*i)   
    }
    */
}

//***************************[ PORTS ]****************************************************
void Port_Init(void)
{
    //CAUTION:Follow the configuration order for setting the ports. 
    // 1) setting value(GPnDAT) 
    // 2) setting control register  (GPnCON)
    // 3) configure pull-up resistor(GPnUP)  

    //32bit data bus configuration  
    //*** PORT A GROUP
    //Ports  : GPA22 GPA21  GPA20 GPA19 GPA18 GPA17 GPA16 GPA15 GPA14 GPA13 GPA12  
    //Signal : nFCE nRSTOUT nFRE   nFWE  ALE   CLE  nGCS5 nGCS4 nGCS3 nGCS2 nGCS1 
    //Binary :  1     1      1  , 1   1   1    1   ,  1     1     1     1
    //Ports  : GPA11   GPA10  GPA9   GPA8   GPA7   GPA6   GPA5   GPA4   GPA3   GPA2   GPA1  GPA0
    //Signal : ADDR26 ADDR25 ADDR24 ADDR23 ADDR22 ADDR21 ADDR20 ADDR19 ADDR18 ADDR17 ADDR16 ADDR0 
    //Binary :  1       1      1      1   , 1       1      1      1   ,  1       1     1      1         
    rGPACON = 0x7fffff; 

    //**** PORT B GROUP
    //Ports  : GPB10    GPB9    GPB8    GPB7    GPB6     GPB5    GPB4   GPB3   GPB2     GPB1      GPB0
    //Signal : nXDREQ0 nXDACK0 nXDREQ1 nXDACK1 nSS_KBD nDIS_OFF L3CLOCK L3DATA L3MODE nIrDATXDEN Keyboard
    //Setting: INPUT  OUTPUT   INPUT  OUTPUT   INPUT   OUTPUT   OUTPUT OUTPUT OUTPUT   OUTPUT    OUTPUT 
    //Binary :   00  ,  01       00  ,   01      00   ,  01       01  ,   01     01   ,  01        01  
 //   rGPBCON = 0x044555;
 //   rGPBUP  = 0x7ff;     // The pull up function is disabled GPB[10:0]
    //rGPBCON = 0x15401;
	///////rGPBDAT &= ~0x1;
   
    //*** PORT C GROUP
    //Ports  : GPC15 GPC14 GPC13 GPC12 GPC11 GPC10 GPC9 GPC8  GPC7   GPC6   GPC5 GPC4 GPC3  GPC2  GPC1 GPC0
    //Signal : VD7   VD6   VD5   VD4   VD3   VD2   VD1  VD0 LCDVF2 LCDVF1 LCDVF0 VM VFRAME VLINE VCLK LEND  
    //Binary :  10   10  , 10    10  , 10    10  , 10   10  , 10     10  ,  10   10 , 10     10 , 10   10
    //rGPCCON = 0xaaaa6aaa;       
    //rGPCCON  = 0xaaaaaaaa;
    //rGPCUP  = 0xffff;     // The pull up function is disabled GPC[15:0] 

    //*** PORT D GROUP
    //Ports  : GPD15 GPD14 GPD13 GPD12 GPD11 GPD10 GPD9 GPD8 GPD7 GPD6 GPD5 GPD4 GPD3 GPD2 GPD1 GPD0
    //Signal : VD23  VD22  VD21  VD20  VD19  VD18  VD17 VD16 VD15 VD14 VD13 VD12 VD11 VD10 VD9  VD8
    //Binary : 10    10  , 10    10  , 10    10  , 10   10 , 10   10 , 10   10 , 10   10 ,10   10
    rGPDCON = 0xaaaaaaaa;       
    rGPDUP  = 0xffff;     // The pull up function is disabled GPD[15:0]

    //*** PORT E GROUP
    //Ports  : GPE15  GPE14 GPE13   GPE12   GPE11   GPE10   GPE9    GPE8     GPE7  GPE6  GPE5   GPE4  
    //Signal : IICSDA IICSCL SPICLK SPIMOSI SPIMISO SDDATA3 SDDATA2 SDDATA1 SDDATA0 SDCMD SDCLK I2SSDO 
    //Binary :  10     10  ,  10      10  ,  10      10   ,  10      10   ,   10    10  , 10     10  ,     
    //-------------------------------------------------------------------------------------------------------
    //Ports  :  GPE3   GPE2  GPE1    GPE0    
    //Signal : I2SSDI CDCLK I2SSCLK I2SLRCK     
    //Binary :  10     10  ,  10      10 
    rGPECON = 0xaaaaaaaa;       
    rGPEUP  = 0xffff;     // The pull up function is disabled GPE[15:0]

	
    //*** PORT F GROUP
    //Ports  : GPF7   GPF6   GPF5   GPF4      GPF3     GPF2  GPF1   GPF0
    //Signal : nLED_8 nLED_4 nLED_2 nLED_1 nIRQ_PCMCIA EINT2 KBDINT EINT0
    //Setting: Output Output Output Output    EINT3    EINT2 EINT1  EINT0
    //Binary :  01      01 ,  01     01  ,     10       10  , 10     10
    //rGPFCON = 0x55aa;
    rGPFUP  = 0xff;     // The pull up function is disabled GPF[7:0]

    //*** PORT G GROUP
    //Ports  : GPG15 GPG14 GPG13 GPG12 GPG11    GPG10    GPG9     GPG8     GPG7      GPG6    
    //Signal : nYPON  YMON nXPON XMON  EINT19 DMAMODE1 DMAMODE0 DMASTART KBDSPICLK KBDSPIMOSI
    //Setting: nYPON  YMON nXPON XMON  EINT19  Output   Output   Output   SPICLK1    SPIMOSI1
    //Binary :   11    11 , 11    11  , 10      01    ,   01       01   ,    11         11
    //-----------------------------------------------------------------------------------------
    //Ports  :    GPG5       GPG4    GPG3    GPG2    GPG1    GPG0    
    //Signal : KBDSPIMISO LCD_PWREN EINT11 nSS_SPI IRQ_LAN IRQ_PCMCIA
    //Setting:  SPIMISO1  LCD_PWRDN EINT11   nSS0   EINT9    EINT8
    //Binary :     11         11   ,  10      11  ,  10        10
    //rGPGCON = 0xff95ffba;
    //rGPGCON = 0xFD95ffba; // gpg12 output.
    rGPGCON = 0xFD000000; // gpg12 output.
    rGPGUP  = 0xffff;    // The pull up function is disabled GPG[15:0]

    //*** PORT H GROUP
    //Ports  :  GPH10    GPH9  GPH8 GPH7  GPH6  GPH5 GPH4 GPH3 GPH2 GPH1  GPH0 
    //Signal : INPUT 	INPUT  UCLK nCTS1 nRTS1 RXD1 TXD1 RXD0 TXD0 nRTS0 nCTS0
    //Binary :   00   ,  00     10 , 11    11  , 10   10 , 10   10 , 10    10
    rGPHCON = 0x02faaa;
    //rGPHCON = 0x02f55a;
    rGPHUP  = 0x7ff;    // The pull up function is disabled GPH[10:0]
    

    //External interrupt will be falling edge triggered. 
    rEXTINT0 = 0x22222222;    // EINT[7:0]
    rEXTINT1 = 0x22222222;    // EINT[15:8]
    rEXTINT2 = 0x22222222;    // EINT[23:16]
    
}

//***************************[ UART ]******************************

void Uart_Init(void)
{
    U32 Baudrate;// (PCLK/16./115200+0.5) 

//    if(pclk == 0)    pclk    = PCLK;
	if(rGSTATUS1&0xFFFF0000 == 0x32410000) {
		Baudrate = (S3C2410PCLK/16 + BAUDRATE/2)/BAUDRATE;
	} else {
		Baudrate = (S3C2440PCLK/16 + BAUDRATE/2)/BAUDRATE;	
	}
    rUFCON0 = 0x0;   //UART channel 0 FIFO control register, FIFO disable
//    rUFCON1 = 0x0;   //UART channel 1 FIFO control register, FIFO disable
//    rUFCON2 = 0x0;   //UART channel 2 FIFO control register, FIFO disable
    rUMCON0 = 0x0;   //UART chaneel 0 MODEM control register, AFC disable
//    rUMCON1 = 0x0;   //UART chaneel 1 MODEM control register, AFC disable

	//UART0
    rULCON0 = 0x3;   //Line control register : Normal,No parity,1 stop,8 bits
     //    [10]       [9]     [8]        [7]        [6]      [5]         [4]           [3:2]        [1:0]
     // Clock Sel,  Tx Int,  Rx Int, Rx Time Out, Rx err, Loop-back, Send break,  Transmit Mode, Receive Mode
     //     0          1       0    ,     0          1        0           0     ,       01          01
     //   PCLK       Level    Pulse    Disable    Generate  Normal      Normal        Interrupt or Polling
    rUCON0  = 0x245;   // Control register
    rUBRDIV0=( (int)(Baudrate) -1 );   //Baud rate divisior register 0
    

	Delay(100);
}



//=====================================================================
char Uart_Getch(void)
{
	while(!(rUTRSTAT0 & 0x1)); //Receive data ready
	return RdURXH0();


}


//=====================================================================
void Uart_SendByte(int data)
{
	if(data=='\n')        {
		while(!(rUTRSTAT0 & 0x2));
		Delay(10);                 //because the slow response of hyper_terminal 
		WrUTXH0('\r');
	}

	while(!(rUTRSTAT0 & 0x2));   //Wait until THR is empty.
	Delay(10);
	WrUTXH0(data);
}               

//====================================================================
void Uart_SendString(char *pt)
{
    while(*pt)
        Uart_SendByte(*pt++);
}

void Uart_SendString_N(char *pt, int len)
{
    while(len--)
      Uart_SendByte(*pt++);
}

