;=========================================
; NAME: 2442INIT.S
; DESC: C start up codes
;       Configure memory, ISR ,stacks
;	Initialize C-variables
; HISTORY:
; 2002.02.25:kwtark: ver 0.0
; 2002.03.20:purnnamu: Add some functions for testing STOP,Sleep mode
; 2003.03.14:DonGo: Modified for 2440.
;=========================================

  IMPORT  __use_no_semihosting_swi
	GET memcfg.inc
	GET 244X_addr.inc

_STACK_BASEADDRESS	EQU 0x34000000

SVCStack	EQU	_STACK_BASEADDRESS		;0x33ff5800 ~


;---------------------------------------------------------------------------


	IMPORT  |Image$$RO$$Limit|  ; End of ROM code (=start of ROM data)
	IMPORT  |Image$$RW$$Base|   ; Base of RAM to initialise
	IMPORT  |Image$$ZI$$Base|   ; Base and limit of area
	IMPORT  |Image$$ZI$$Limit|  ; to zero initialise

	IMPORT  Main    ; The main entry of mon program

	AREA    Init,CODE,READONLY

	ENTRY

	b	ResetHandler
	b	.	;handler for Undefined mode
	b	.	;handler for SWI interrupt
	b	.	;handler for PAbort
	b	.	;handler for DAbort
	b	.	;reserved
	b	.	;handler for IRQ interrupt
	b	.	;handler for FIQ interrupt


;======================================================================================
; ENTRY
;======================================================================================
ResetHandler

	ldr	r0,=WTCON       ;watch dog disable
	ldr	r1,=0x0
	str	r1,[r0]

	ldr	r0,=INTMSK
	ldr	r1,=0xffffffff  ;all interrupt disable
	str	r1,[r0]

	ldr	r0,=INTSUBMSK
	ldr	r1,=0x7ff	;all sub interrupt disable
	str	r1,[r0]


	ldr	r0,=GPGCON
	ldr	r1,=0xFD95ffba	; set output
	str	r1,[r0]
	ldr	r0,=GPGDAT		;set high.
	ldr	r1,=0x1000  
	str	r1,[r0]
	ldr r0,=GPGUP
	ldr r1,=0xffff
	str r1,[r0]
	;To reduce PLL lock time, adjust the LOCKTIME register.
	ldr	r0,=LOCKTIME
	ldr	r1,=0xffffff
	str	r1,[r0]

   
	ldr	r0,=CLKDIVN
	ldr	r1,=7
	str	r1,[r0]

	;Configure UPLL
	ldr	r0,=UPLLCON
	ldr	r1,=((56<<12)+(2<<4)+2)  ;Fin=12MHz,Fout=48MHz
	str	r1,[r0]
	nop	; Caution: After UPLL setting, at least 7-clocks delay must be inserted for setting hardware be completed.
	nop
	nop
	nop
	nop
	nop
	nop

	;Configure MPLL
	ldr	r0,=MPLLCON
	ldr	r1,=((92<<12)+(1<<4)+1)
	str	r1,[r0]
	

; :::::::::::::::::::::::::::::::::::::::::::::
;           BEGIN: Power Management 
; - - - - - - - - - - - - - - - - - - - - - - -
	ldr	r1,=GSTATUS2
	ldr	r0,[r1]
	tst	r0,#0x2
	beq	%F4
	
	ldr 	r1, =MISCCR        	; MISCCR's Bit 17, 18, 19 -> 0
	ldr	r0, [r1]                ; I don't know why, Just fallow Sample Code.
	bic	r0, r0, #(3 << 17)      ; SCLK0:0->SCLK, SCLK1:0->SCLK, SCKE:L->H
	str	r0, [r1]

	; Set memory control registers
	add	r0, pc, #SMRDATA - (. + 8)
	ldr	r1, =BWSCON	; BWSCON Address
	add	r2, r0, #52	; End address of SMRDATA
loop10
	ldr	r3, [r0], #4
	str	r3, [r1], #4
	cmp	r2, r0
	bne	loop10

	mov	r1, #256
loop11
	subs	r1, r1, #1		; wait until the SelfRefresh is released.
	bne	loop11


;	ldr r1,=GSTATUS3 		;GSTATUS3 has the start address just after SLEEP wake-up
;  α֤ȥǰڴȷ!
	ldr	r2, =0x201000		; offset into the RAM 
	add	r2, r2, #0x30000000	; add physical base
	mov     pc, r2			;  & jump to StartUp address
	nop
	nop
	nop
	b .

; - - - - - - - - - - - - - - - - - - - - - - -
;           END: Power Management 
; :::::::::::::::::::::::::::::::::::::::::::::
4	
    	; Configure memory controller
    	;ldr    r0,=SMRDATA
    	add     r0, pc, #SMRDATA - (. + 8)

	ldr	r1,=BWSCON	;BWSCON Address
	add	r2, r0, #52	;End address of SMRDATA
0
	ldr	r3, [r0], #4
	str	r3, [r1], #4
	cmp	r2, r0
	bne	%B0

   	;Initialize stacks
	;bl	InitStacks
	ldr	sp,=SVCStack		; 


	;Copy and paste RW data/zero initialized data
	ldr	r0, =|Image$$RO$$Limit| ; Get pointer to ROM data
	ldr	r1, =|Image$$RW$$Base|  ; and RAM copy
	ldr	r3, =|Image$$ZI$$Base|

	;Zero init base => top of initialised data
	cmp	r0, r1      ; Check that they are different
	beq	%F2
1
	cmp	r1, r3      ; Copy init data
	ldrcc	r2, [r0], #4    ;--> LDRCC r2, [r0] + ADD r0, r0, #4
	strcc	r2, [r1], #4    ;--> STRCC r2, [r1] + ADD r1, r1, #4
	bcc	%B1
2
	ldr	r1, =|Image$$ZI$$Limit| ; Top of zero init segment
	mov	r2, #0
3
	cmp	r3, r1      ; Zero init
	strcc	r2, [r3], #4
	bcc	%B3


	bl	Main	;Don't use main() because ......
	b	.

;function initializing stacks
;InitStacks
	;Don't use DRAM,such as stmfd,ldmfd......
	;SVCstack is initialized before
	;Under toolkit ver 2.5, 'msr cpsr,r1' can be used instead of 'msr cpsr_cxsf,r1'
;	mrs	r0,cpsr
;	bic	r0,r0,#MODEMASK
;	orr	r1,r0,#UNDEFMODE|NOINT
;	msr	cpsr_cxsf,r1		;UndefMode
;	ldr	sp,=UndefStack		; UndefStack=0x33FF_5C00

;	orr	r1,r0,#ABORTMODE|NOINT
;	msr	cpsr_cxsf,r1		;AbortMode
;	ldr	sp,=AbortStack		; AbortStack=0x33FF_6000

;	orr	r1,r0,#IRQMODE|NOINT
;	msr	cpsr_cxsf,r1		;IRQMode
;	ldr	sp,=IRQStack		; IRQStack=0x33FF_7000

;	orr	r1,r0,#FIQMODE|NOINT
;	msr	cpsr_cxsf,r1		;FIQMode
;	ldr	sp,=FIQStack		; FIQStack=0x33FF_8000

;	bic	r0,r0,#MODEMASK|NOINT
;	orr	r1,r0,#SVCMODE
;	msr	cpsr_cxsf,r1		;SVCMode
;	ldr	sp,=SVCStack		; SVCStack=0x33FF_5800

	;USER mode has not be initialized.

;	mov	pc,lr
	;The LR register won't be valid if the current mode is not SVC mode.


	LTORG

SMRDATA DATA
; Memory configuration should be optimized for best performance
; The following parameter is not optimized.
; Memory access cycle parameter strategy
; 1) The memory settings is  safe parameters even at HCLK=75Mhz.
; 2) SDRAM refresh period is for HCLK<=75Mhz.

	DCD (0+(B1_BWSCON<<4)+(B2_BWSCON<<8)+(B3_BWSCON<<12)+(B4_BWSCON<<16)+(B5_BWSCON<<20)+(B6_BWSCON<<24)+(B7_BWSCON<<28))
	DCD ((B0_Tacs<<13)+(B0_Tcos<<11)+(B0_Tacc<<8)+(B0_Tcoh<<6)+(B0_Tah<<4)+(B0_Tacp<<2)+(B0_PMC))   ;GCS0
	DCD ((B1_Tacs<<13)+(B1_Tcos<<11)+(B1_Tacc<<8)+(B1_Tcoh<<6)+(B1_Tah<<4)+(B1_Tacp<<2)+(B1_PMC))   ;GCS1
	DCD ((B2_Tacs<<13)+(B2_Tcos<<11)+(B2_Tacc<<8)+(B2_Tcoh<<6)+(B2_Tah<<4)+(B2_Tacp<<2)+(B2_PMC))   ;GCS2
	DCD ((B3_Tacs<<13)+(B3_Tcos<<11)+(B3_Tacc<<8)+(B3_Tcoh<<6)+(B3_Tah<<4)+(B3_Tacp<<2)+(B3_PMC))   ;GCS3
	DCD ((B4_Tacs<<13)+(B4_Tcos<<11)+(B4_Tacc<<8)+(B4_Tcoh<<6)+(B4_Tah<<4)+(B4_Tacp<<2)+(B4_PMC))   ;GCS4
	DCD ((B5_Tacs<<13)+(B5_Tcos<<11)+(B5_Tacc<<8)+(B5_Tcoh<<6)+(B5_Tah<<4)+(B5_Tacp<<2)+(B5_PMC))   ;GCS5
	DCD ((B6_MT<<15)+(B6_Trcd<<2)+(B6_SCAN))    ;GCS6
	DCD ((B7_MT<<15)+(B7_Trcd<<2)+(B7_SCAN))    ;GCS7
	DCD ((REFEN<<23)+(TREFMD<<22)+(Trp<<20)+(Tsrc<<18)+(Tchr<<16)+REFCNT)
	;;;
	;   DCD 0x00aC03F4 ;refresh
	DCD 0xB2	    ;SCLK power saving mode, BANKSIZE 128M/128M


	DCD 0x30	    ;MRSR6 CL=3clk
	DCD 0x30	    ;MRSR7 CL=3clk

	ALIGN

;	AREA RamData, DATA, READWRITE

;	^   _ISR_STARTADDRESS		; MAP  _ISR_STARTADDRESS=0x33FF_FF00
;vCLKDIV_VAL	DCD	CLKDIV_VAL
;vMPLL_VAL	DCD	((M_MDIV<<12)+(M_PDIV<<4)+M_SDIV)
;vUPLL_VAL	DCD	((U_MDIV<<12)+(U_PDIV<<4)+U_SDIV)


	END

