/************************************************
 * NAME    : nboot.c
 * DESC    : 
 * History : 2008.12.25 ver 0.0
************************************************/
#include <stdlib.h>
#include <string.h>
#include "def.h"
#include "option.h"
#include "244X_addr.h"
#include "Nand.h"
#include "244X_lib.h"

#define TOC_BLOCK                   (2)
#define SECTOR_PER_BLOCK			256
#define SECTOR_SIZE					512

#define DOWNLOAD_ADDRESS 0x30138000

void (*run)(void)=(void (*)(void))(DOWNLOAD_ADDRESS);


TOC *pToc = (TOC*)(0x33F00000);


// Progress Bar
static unsigned Total;
static unsigned Retail;
static unsigned char *CurrAddrInLCD;

static void ProgressBarInit(unsigned total)
{
	Total  = total;
	Retail = 0;
	CurrAddrInLCD = (unsigned char *)LCD_ADDR + PROGRESS_BAR_TOP * LCD_WIDTH * 2 + PROGRESS_BAR_LEFT * 2;
}

static void ProgressBarStep(void)
{
	Retail += PROGRESS_BAR_WIDTH;
	while (Retail > Total) {
		int y;
		unsigned short *p = (unsigned short *)CurrAddrInLCD;
		// draw a vertial line, with 1 width
		Retail -= Total;
		
		for (y = 0; y < PROGRESS_BAR_HEIGHT; y++) {
			*p = PROGRESS_BAR_COLOR;
			p += LCD_WIDTH;
		}
		CurrAddrInLCD += 2;
	}
}

// Loading
char ReadImageFromNand(unsigned int dwEntry)
{
    volatile unsigned int dwSectorsNeeded;
    volatile unsigned int dwSector, dwLength;         // Start Sector & Length
    volatile unsigned int dwRAM, i;


	  NF_ReadSector(TOC_BLOCK*SECTOR_PER_BLOCK, (U8*)pToc);
	
    if (!VALID_TOC(pToc))
        return 0x32;
    
    if ( !(pToc->id[dwEntry].dwImageType & IMAGE_TYPE_RAMIMAGE) ) {
        return 0x33;
    }

    dwSectorsNeeded = pToc->id[dwEntry].dwTtlSectors;
	
    dwRAM    = VIRTUAL_TO_PHYSICAL(pToc->id[dwEntry].dwLoadAddress);

    run = (void (*)(void))(pToc->id[dwEntry].dwJumpAddress ? VIRTUAL_TO_PHYSICAL(pToc->id[dwEntry].dwJumpAddress) :
                                               VIRTUAL_TO_PHYSICAL(pToc->id[dwEntry].dwLoadAddress));
                                               

    i = 0;
	  while (dwSectorsNeeded && i < MAX_SG_SECTORS)
	  {
        dwSector = pToc->id[dwEntry].sgList[i].dwSector;
        dwLength = pToc->id[dwEntry].sgList[i].dwLength;

		ProgressBarInit(dwLength);
        // read each sg segment
        while (dwLength) {
            if (dwSector % SECTOR_PER_BLOCK == 0) {

                if (! NF_IsBadBlock(dwSector/SECTOR_PER_BLOCK)) {
                    dwSector += SECTOR_PER_BLOCK;
                    continue;                        
                }


            }
            
            if(!NF_ReadSector(dwSector, (unsigned char*)dwRAM)){
                    Uart_SendString("\nCannot load Windows CE\n");
                    while (1);                        
            }            

   		    dwSector++;
   		    dwLength--;
            dwRAM += SECTOR_SIZE;
            ProgressBarStep();
        }

        dwSectorsNeeded -= pToc->id[dwEntry].sgList[i].dwLength;
        i++;
    }

	  return 0;
}


#define pBSPArgs                    ((unsigned char *) 0x30020800)


// Picture in Nand
#define PIC_MAX_SIZE ( ( (1024 * 768 * 2 + 8 - 1) / (128 * 1024) + 1 ) * (128 * 1024) )
#define PIC_NAND_POS 0x60000
static __inline void LoadPicture(void)
{
	U16 Buf[PIC_MAX_SIZE / 2];
	U8  *ImgData = (unsigned char*)(Buf+4); // Start at 8 
	U16 Width, Height;
	U32 y;
	unsigned int i;
	
	U8 *lcd, *pic;

    // try
	NF_ReadSector(PIC_NAND_POS / SECTOR_SIZE, (unsigned char *)Buf);


	// Get Magic number, width, height
	if (Buf[0] != 0x4146 || Buf[1] != 0x4C57) {
		Uart_SendString("\r\nno logo picture");
		return;
	}
	
	Width  = Buf[2];
	Height = Buf[3];
	if (LOGO_POS_LEFT + Width > LCD_WIDTH || LOGO_POS_TOP + Height > LCD_HEIGHT) {
		//Uart_SendString("\r\nPicuture or pos out of range of LCD");
		return;
	}
	if (Width * Height * 2 + 8 > PIC_MAX_SIZE) {
	    //Uart_SendString("\r\nlogo too big? not flashed by official FriendlyARM?");
		return;
	}
	
	//Uart_SendString("\r\nReading");
	// Get rest picture data
	for (i = SECTOR_SIZE; i < Height * Width * 2 + 8; i+= SECTOR_SIZE) {
		NF_ReadSector( (PIC_NAND_POS + i) / SECTOR_SIZE, ((unsigned char *)Buf) + i);
	}
	//Uart_SendString("\r\nRead");

	// draw it
	y = Height;
	lcd = (U8 *)LCD_ADDR + LCD_WIDTH * LOGO_POS_TOP * 2 + LOGO_POS_LEFT * 2;
	pic = ImgData;
	while (y--){
		Memcpy((char *)lcd, (char *)pic, Width * 2);
		lcd += LCD_WIDTH * 2;
		pic += Width * 2;
	}
}

static void InitLCD(void)
{
	unsigned i = LCD_WIDTH * LCD_HEIGHT;
	
	// Fill LCD with background color
	while (i--) {
		((unsigned short *)LCD_ADDR)[i] = BACKGROUND_COLOR;
	}
    LoadPicture();

	// Open LCD Device	
	rLCDCON1 = (LCD_PIXCLOCK << 8) | (3 <<  5) | (12 << 1);
   	rLCDCON2 = (LCD_UPPER_MARGIN << 24) | ((LCD_HEIGHT - 1) << 14) | (LCD_LOWER_MARGIN << 6) | (LCD_VSYNC_LEN << 0);
   	rLCDCON3 = (LCD_RIGHT_MARGIN << 19) | ((LCD_WIDTH  - 1) <<  8) | (LCD_LEFT_MARGIN << 0);
   	rLCDCON4 = (13 <<  8) | (LCD_HSYNC_LEN << 0);

#if !defined(LCD_CON5)
#    define LCD_CON5 ((1<<11) | (1 << 9) | (1 << 8) | (1 << 3) | (1 << 0))
#endif
    rLCDCON5   =  LCD_CON5;

    rLCDSADDR1 = ((LCD_ADDR >> 22) << 21) | ((M5D(LCD_ADDR >> 1)) <<  0);
    rLCDSADDR2 = M5D((LCD_ADDR + LCD_WIDTH * LCD_HEIGHT * 2) >> 1);
    rLCDSADDR3 = LCD_WIDTH;        

    rLCDINTMSK |= 3;
  	rTCONSEL   &= (~7);
 

	rGPCCON = 0xaaaaaaaaU;
	rGPCUP  = 0xffffU; 
   	rTPAL     = 0x0;
   	rTCONSEL &= ~((1<<4) | 1);
    rLCDCON1  |= 1;


    rGPGCON = 1<<8;
    rGPGDAT = 1<<4;
    
}


//
void Main(void)
{
    char result;
	
	MMU_EnableICache();
	MMU_EnableDCache();
	
	Port_Init();
	Uart_Init();
	Nand_Init();

    //Memset((char*)0x30000000, 0,  3U * 512 * 1024);
    //Memset((char*)0x30000000 + 32U * 1024 * 1024, 0,  128U * 1024U);
    Memset((char*)pBSPArgs, 0, 1024);
    Uart_SendString("load Windows CE Image..\r\n");
    //for(;;);
 
 	InitLCD();
    result = ReadImageFromNand(1);


	  if (run)   run();

	  // shouldn't be here.

	  for (;;);
}
