#include <string.h>
#include <stdlib.h>
#include "def.h"
#include "option.h"
#include "244X_addr.h"
#include "244X_lib.h"
#include "nand.h"

#define NF_MECC_UnLock()    {rNFCONT&=~(1<<5);}
#define NF_MECC_Lock()      {rNFCONT|=(1<<5);}

#define NF_CMD(cmd)			{rNFCMD=cmd;}
#define NF_ADDR(addr)		{rNFADDR=addr;}	
#define NF_nFCE_L()			{rNFCONT&=~(1<<1);}
#define NF_nFCE_H()			{rNFCONT|=(1<<1);}
#define NF_RSTECC()			{rNFCONT|=(1<<4);}
#define NF_RDDATA() 		(rNFDATA)
#define NF_RDDATA8() 		((*(volatile unsigned char*)0x4E000010) )

#define NF_WRDATA(data) 	{rNFDATA=data;}

#define NF_WAITRB()    		{while(!(rNFSTAT&(1<<1)));} 
	   						 //wait tWB and check F_RNB pin.
// RnB Signal
#define NF_CLEAR_RB()    	{rNFSTAT |= (1<<2);}	// Have write '1' to clear this bit.
#define NF_DETECT_RB()    	{while(!(rNFSTAT&(1<<2)));}

#define ID_K9S1208V0M		0xec76
#define ID_K9K2G16U0M		0xecca


#define TACLS		0	// 1-clk(0ns) 
#define TWRPH0		6	// 3-clk(25ns)
#define TWRPH1		2	// 1-clk(10ns)  //TACLS+TWRPH0+TWRPH1>=50ns

void __RdPage512(U8 *buf);

static int bLargeBlock;

static void Nand_Reset(void)
{
  volatile int i;
   
	NF_nFCE_L();
	NF_CLEAR_RB();
	for (i=0; i<10; i++);
	NF_CMD(0xFF);	//reset command
	NF_DETECT_RB();
	NF_nFCE_H();

}


U32 NF8_CheckId(void)
{
	int i;
	U8 Mid, Did, DontCare, id4th;  		

	NF_nFCE_L();
	
 	NF_CMD(0x90);
	NF_ADDR(0x0);
	for (i=0; i<100; i++);

	Mid 		= NF_RDDATA8();
	Did   		= NF_RDDATA8();
	DontCare 	= NF_RDDATA8();
	id4th 		= NF_RDDATA8();	

	NF_nFCE_H();
	
	switch(Did) {
	case 0x76:
		bLargeBlock = 0;
		break;
	case 0xF1:
	case 0xD3:
	case 0xDA:
	case 0xDC:
		bLargeBlock = 1;
		break;
	default:
		Uart_SendString("Unknown NAND type\r\n");
		for(;;);
	}

	return (U32)((Mid<<24)|(Did<<16)|(DontCare<<8)|id4th);
}

void Nand_Init(void)
{
	rNFCONF = (TACLS<<12)|(TWRPH0<<8)|(TWRPH1<<4)|(0<<0);	
	rNFCONT = (0<<13)|(0<<12)|(0<<10)|(0<<9)|(0<<8)|(0<<6)|(0<<5)|(1<<4)|(1<<1)|(1<<0);
	rNFSTAT = 0;
	Nand_Reset();
	NF8_CheckId();
}




static int Nand_IsBadBlockPage512(U32 block)
{
    U32 	Page;
	U8 		BAD;
	
    volatile int time;
    for (time=0; time<8; time++) {
	      Page = (block<<8) + time*32;

        Nand_Reset();
    		
        NF_nFCE_L();
	      NF_CLEAR_RB();
  	    NF_CMD(0x50);		 	    
  	    NF_ADDR(5);	    
        NF_ADDR(Page&0xff);	 		
        NF_ADDR((Page>>8)&0xff);	
        NF_ADDR((Page>>16)&0xff);   
        NF_DETECT_RB();	 			

        BAD = NF_RDDATA8();
             
        NF_nFCE_H();    

        if(BAD != 0xff) 	{
          	  return FAIL;
        }
    }        
      
    return OK;
    
}

static int Nand_IsBadBlockPage2048(U32 block)
{
    U32 	Page;
	U8 		BAD;
	
   	Page = block << 6;      // block * 64
   	NF_nFCE_L();
    NF_CLEAR_RB();
   	NF_CMD(0x00);
	NF_ADDR(0);
	NF_ADDR((2048>>8)&0xff);
    NF_ADDR(Page&0xff);	 		// The mark of bad block is in 0 page
    NF_ADDR((Page>>8)&0xff);	// For block number A[24:17]
    NF_ADDR((Page>>16)&0xff);   // For block number A[25]
    NF_CMD(0x30);        
   	NF_DETECT_RB();	 			// Wait tR(max 12us)

   	BAD = NF_RDDATA8();
             
   	NF_nFCE_H();    

    if(BAD != 0xff) 	{
   		return FAIL;
    }

    return OK;
    
}

int Nand_IsBadBlock(U32 block)
{
	switch(bLargeBlock) {
	case 0:
		return Nand_IsBadBlockPage512(block);
	case 1:
		return Nand_IsBadBlockPage2048(block);
	default:
		for(;;);
	}

}

static int Nand_ReadSectorPage512(U32 sector, U8 *buffer)
{
	U32 ecc, i;
	U8 se[16], ecc0, ecc1, ecc2;

  	Nand_Reset();
	NF_RSTECC();
	NF_MECC_UnLock();
    
	NF_nFCE_L();    

	NF_CLEAR_RB();
	NF_CMD(0x00);

    NF_ADDR(0x00);
	NF_ADDR(sector&0xff);
	NF_ADDR((sector>>8)&0xff);
	NF_ADDR((sector>>16)&0xff);
	 
	NF_DETECT_RB();	

    __RdPage512(buffer);
  
    NF_MECC_Lock();

	ecc  = rNFMECC0;
	ecc0 = (U8)(ecc &0xff);
	ecc1 = (U8)((ecc>>8) & 0xff);
	ecc2 = (U8)((ecc>>16) & 0xff);
		
	for(i=0;i<16;i++)   {
			se[i]=NF_RDDATA8();	// Read spare array
	}

	NF_nFCE_H();    

	if(ecc0==se[8] && ecc1==se[9] && ecc2==se[10] )
	  return TRUE;

  return FAIL;
}

static int Nand_ReadSectorPage2048(U32 sector, U8 *buffer)
{
	U32 page, data_addr;

    //Nand_Reset();
	//NF_RSTECC();    					// Initialize ECC
	//NF_MECC_UnLock();
    
	NF_nFCE_L();    

	NF_CLEAR_RB();
	NF_CMD(0x00);						// Read command

	page = sector/4;
	data_addr = 512 *(sector%4);
	NF_ADDR(data_addr&0xff);
	NF_ADDR((data_addr>>8)&0xff);
	NF_ADDR(page&0xff);
	NF_ADDR((page>>8)&0xff);
	NF_ADDR((page>>16)&0xff);
	NF_CMD(0x30);
	
	 
	NF_DETECT_RB();	

    __RdPage512(buffer);

	NF_nFCE_H();    

   	return OK;
}


int Nand_ReadSector(U32 sector, U8 *buffer)
{
	switch(bLargeBlock) {
	case 0:
		return Nand_ReadSectorPage512(sector, buffer);
	case 1:
		return Nand_ReadSectorPage2048(sector, buffer);
	default:
		for(;;);
	}
}
